--[[ $Id: ChannelClean.lua 11249 2006-09-18 12:43:45Z hshh $ ]]--
-- ACE2 init
local L = AceLibrary("AceLocale-2.1"):GetInstance("ChannelClean", true)
ChannelClean = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceHook-2.0", "AceConsole-2.0")

function ChannelClean:OnInitialize()
	self.HookedChannels={
		"CHAT_MSG_CHANNEL",
		"CHAT_MSG_SAY",
		"CHAT_MSG_WHISPER",
		"CHAT_MSG_YELL",
		"CHAT_MSG_GUILD",
		"CHAT_MSG_PARTY",
		"CHAT_MSG_RAID",
		"CHAT_MSG_SYSTEM"
	}
	self.lastNotice = {}
	self.trigger_temp = {}

	-- init saved variables
	self:RegisterDB("ChannelCleanDB")
	self:RegisterDefaults('account', {
		toggle = false,
		notice = true,

		CHAT_MSG_CHANNEL = true,
		CHAT_MSG_SAY = true,
		CHAT_MSG_WHISPER = true,
		CHAT_MSG_YELL = true,
		CHAT_MSG_GUILD = false,
		CHAT_MSG_PARTY = false,
		CHAT_MSG_RAID = false,
		CHAT_MSG_SYSTEM = false,
		
		player = {},
		message = {},
		trigger = {},
		trigger_mode = "session"
	})

	-- register console slash commands
	self:RegisterChatCommand(
		{ "/channelclean", "/chc" }, 
		{
			type = "group",
			args = {
				reset = {
					type = 'execute',
					name = "reset",
					desc = L["Reset all settings. Clear all block rules."],
					func = function()
						self:ResetDB("account")
						self:OnDisable()
						self:OnEnable()
						self.lastNotice = {}
						self.trigger_temp = {}
						self:Print(L["Reset to default settings, all block rules are cleared."])
					end
				},
				toggle = {
					type = "toggle",
					name = "toggle",
					desc = L["Toggle ChannelClean enabled or disabled."],
					get = function() return self.db.account.toggle end,
					set = function(c)
						self.db.account.toggle = c
						if self.db.account.toggle then
							self:Print(L["ChannelClean is enabled."])
						else
							self:Print(L["ChannelClean is disabled."])
						end
						self:OnDisable()
						self:OnEnable()
					end,
				},
				notice = {
					type = "toggle",
					name = "notice",
					desc = L["Toggle blocked notice enabled or disabled."],
					get = function() return self.db.account.notice end,
					set = function(c)
						self.db.account.notice = c
						if self.db.account.notice then
							self:Print(L["Blocked notice is enabled."])
						else
							self:Print(L["Blocked notice is disabled."])
						end
					end,
				},
				monitor = {
					type = "group",
					name = "monitor",
					desc = L["Setting channel monitor lists."],
					args = {
						channel = {
							type = "toggle",
							name = "channel",
							desc = L["Toggle block chat channel messages."],
							get = function() return self.db.account.CHAT_MSG_CHANNEL end,
							set = function(c) self.db.account.CHAT_MSG_CHANNEL = c end,
						},
						say = {
							type = "toggle",
							name = "say",
							desc = L["Toggle block say messages."],
							get = function() return self.db.account.CHAT_MSG_SAY end,
							set = function(c) self.db.account.CHAT_MSG_SAY = c end,
						},
						whisper = {
							type = "toggle",
							name = "whisper",
							desc = L["Toggle block whisper messages."],
							get = function() return self.db.account.CHAT_MSG_WHISPER end,
							set = function(c) self.db.account.CHAT_MSG_WHISPER = c end,
						},
						yell = {
							type = "toggle",
							name = "yell",
							desc = L["Toggle block yell messages."],
							get = function() return self.db.account.CHAT_MSG_YELL end,
							set = function(c) self.db.account.CHAT_MSG_YELL = c end,
						},
						guild = {
							type = "toggle",
							name = "guild",
							desc = L["Toggle block guild messages."],
							get = function() return self.db.account.CHAT_MSG_GUILD end,
							set = function(c) self.db.account.CHAT_MSG_GUILD = c end,
						},
						party = {
							type = "toggle",
							name = "party",
							desc = L["Toggle block party messages."],
							get = function() return self.db.account.CHAT_MSG_PARTY end,
							set = function(c) self.db.account.CHAT_MSG_PARTY = c end,
						},
						raid = {
							type = "toggle",
							name = "raid",
							desc = L["Toggle block raid messages."],
							get = function() return self.db.account.CHAT_MSG_RAID end,
							set = function(c) self.db.account.CHAT_MSG_RAID = c end,
						},
						system = {
							type = "toggle",
							name = "system",
							desc = L["Toggle block system messages."],
							get = function() return self.db.account.CHAT_MSG_SYSTEM end,
							set = function(c) self.db.account.CHAT_MSG_SYSTEM = c end,
						}
					}
				},
				add = {
					type = "group",
					name = "add",
					desc = L["Add a new rule set."],
					args = {
						message = {
							type = "text",
							name = "message",
							desc = L["Add a message block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:AddRule("message", c) end
						},
						player = {
							type = "text",
							name = "player",
							desc = L["Add a player block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:AddRule("player", c) end
						},
						trigger = {
							type = "text",
							name = "trigger",
							desc = L["Add a trigger block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:AddRule("trigger", c) end
						}
					}
				},
				del = {
					type = "group",
					name = "del",
					desc = L["Delete a exist rule set."],
					args = {
						message = {
							type = "text",
							name = "message",
							desc = L["Delete a message block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:DeleteRule("message", c) end
						},
						player = {
							type = "text",
							name = "player",
							desc = L["Delete a player block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:DeleteRule("player", c) end
						},
						trigger = {
							type = "text",
							name = "trigger",
							desc = L["Delete a trigger block rule."],
							usage = "<block rule>",
							get = false,
							set = function(c) return self:DeleteRule("trigger", c) end
						}
					}
				},
				clear = {
					type = "group",
					name = "clear",
					desc = L["Clear a group rule sets."],
					args = {
						message = {
							type = "execute",
							name = "message",
							desc = L["Clear message block rules."],
							func = function()
								self.db.account.message={}
								self:Print(format(L["RuleClearedFormat"], L["RuleGroup_message"]))
							end
						},
						player = {
							type = "execute",
							name = "player",
							desc = L["Clear player block rules."],
							func = function()
								self.db.account.player={}
								self:Print(format(L["RuleClearedFormat"], L["RuleGroup_player"]))
							end
						},
						trigger = {
							type = "execute",
							name = "trigger",
							desc = L["Clear trigger block rules."],
							func = function()
								self.db.account.trigger={}
								self:Print(format(L["RuleClearedFormat"], L["RuleGroup_trigger"]))
							end
						},
						trigger_session = {
							type = "execute",
							name = "trigger_session",
							desc = L["Clear trigger blocked players."],
							func = function()
								self.trigger_temp = {}
								self:Print(format(L["RuleClearedFormat"], L["RuleGroup_trigger_session"]))
							end
						}
					}
				},
				list = {
					type = "group",
					name = "list",
					desc = L["List a group rule sets."],
					args = {
						message = {
							type = "execute",
							name = "message",
							desc = L["List message block rules."],
							func = function()
								self:Print(format(L["ListFormat"], L["RuleGroup_message"]))
								for k,v in ipairs(self.db.account.message) do
									self:Print(format(L["ListRulesFormat"], k ,v))
								end
							end
						},
						player = {
							type = "execute",
							name = "player",
							desc = L["List player block rules."],
							func = function()
								self:Print(format(L["ListFormat"], L["RuleGroup_player"]))
								for k,v in ipairs(self.db.account.player) do
									self:Print(format(L["ListRulesFormat"], k ,v))
								end
							end
						},
						trigger = {
							type = "execute",
							name = "trigger",
							desc = L["List trigger block rules."],
							func = function()
								self:Print(format(L["ListFormat"], L["RuleGroup_trigger"]))
								for k,v in ipairs(self.db.account.trigger) do
									self:Print(format(L["ListRulesFormat"], k ,v))
								end
							end
						},
						trigger_session = {
							type = "execute",
							name = "trigger_session",
							desc = L["List trigger blocked players."],
							func = function()
								self:Print(format(L["ListFormat"], L["RuleGroup_trigger_session"]))
								for k,v in ipairs(self.trigger_temp) do
									self:Print(format(L["ListRulesFormat"], k ,v))
								end
							end
						}
					}
				},
				triggermode = {
					type = "text",
					name = "triggermode",
					desc = L["Change message block trigger mode."],
					usage = "<session | forever>",
					validate = {"session", "forever"},
					get = function() return self.db.account.trigger_mode end,
					set = function(c) self.db.account.trigger_mode = c end
				}
			}
		}
	)

	-- update old blocklist
	if type(ChannelClean_msglist)=="table" then
		for _,v in ipairs(ChannelClean_msglist) do
			tinsert(self.db.account.message, strlower(v))
		end
		ChannelClean_msglist = nil
		self:Print(L["Converted old message block lists."])
	end
	if type(ChannelClean_ppllist)=="table" then
		for _,v in ipairs(ChannelClean_ppllist) do
			tinsert(self.db.account.player, strlower(v))
		end
		ChannelClean_ppllist = nil
		self:Print(L["Converted old player block lists."])
	end

	-- remove dupe block rules
	self.db.account.player = self:RemoveDupe(self.db.account.player)
	self.db.account.message = self:RemoveDupe(self.db.account.message)
	self.db.account.trigger = self:RemoveDupe(self.db.account.trigger)

	-- hotfix for channel forcus
	self:RegisterEvent("PLAYER_LOGIN")
end

function ChannelClean:PLAYER_LOGIN()
	local chatframe,i
	for i = 1,7 do
		chatframe = getglobal("ChatFrame" .. i)
		if chatframe and chatframe:IsShown() then
			ChatFrame1Tab:Click()
			break
		end
	end
	self:UnregisterAllEvents()
end

function ChannelClean:OnEnable()
	if not self.db.account.toggle then
		return self:OnDisable()
	end
	self:Hook("ChatFrame_OnEvent")
end

function ChannelClean:OnDisable()
	self:UnhookAll()
end

function ChannelClean:ChatFrame_OnEvent(event)
	if not self.db.account.toggle then
		return self:OnDisable()
	end

	-- check event is need to process
	local process
	for _,v in ipairs(self.HookedChannels) do
		if event == v then
			process = true
			if not self.db.account[v] then
				return self.hooks.ChatFrame_OnEvent(event)
			end
		end
	end

	if process then
		local message, player = strlower(arg1), strlower(arg2)
		for k,v in ipairs(self.db.account.message) do
			if strfind(message, v) then
				return self:Notice("message", arg2, k)
			end
		end
		for k,v in ipairs(self.db.account.player) do
			if strfind(player, v) then
				return self:Notice("player", arg2, k)
			end
		end
		for _,v in ipairs(self.trigger_temp) do
			if player == v then
				return self:Notice("trigger_temp", arg2)
			end
		end
		for k,v in ipairs(self.db.account.trigger) do
			if strfind(message, v) then
				-- add to trigger db
				self:AddTrigger(player)
				return self:Notice("trigger", arg2, k)
			end
		end
	end

	return self.hooks.ChatFrame_OnEvent(event)
end

function ChannelClean:Notice(blocktype, blockplayer, blockrule)
	if not self.db.account.notice then
		return
	end

	-- don't fire too much notice
	if self.lastNotice[blockplayer] and time() - self.lastNotice[blockplayer] < 5 then
		return
	end
	self.lastNotice[blockplayer] = time()

	blockmsg = "NOTICE_"..blocktype
	if blocktype == "trigger_temp" then
		self:Print(format(L[blockmsg], blockplayer))
	else
		self:Print(format(L[blockmsg], blockplayer, blockrule))
	end
end

function ChannelClean:AddTrigger(player)
	if self.db.account.trigger_mode == "forever" then
		tinsert(self.db.account.player, player)
	else
		tinsert(self.trigger_temp, player)
	end
end

function ChannelClean:RemoveDupe(blockrules)
	if type(blockrules)~="table" then
		return {}
	end
	local newrules = {}
	for _,check in ipairs(blockrules) do
		if not self:CheckExists(newrules, check) then
			tinsert(newrules, check)
		end
	end
	return newrules
end

function ChannelClean:CheckExists(rules, check)
	for _,vv in ipairs(rules) do
		if vv == check then
			return true
		end
	end
	return false
end

function ChannelClean:AddRule(blocktype, rule)
	rule = strlower(rule)
	-- don't add exists rule
	if self:CheckExists(self.db.account[blocktype], rule) then
		return self:Print(L["Rule is already exist."])
	end
	
	tinsert(self.db.account[blocktype], rule)

	local blocktype = "RuleGroup_"..blocktype
	self:Print(format(L["RuleAddedFormat"], L[blocktype], rule))
end

function ChannelClean:DeleteRule(blocktype, rule)
	rule = strlower(rule)
	local newrule = {}
	for _,v in ipairs(self.db.account[blocktype]) do
		if v and v~=rule then
			tinsert(newrule, v)
		end
	end

	-- delete by rule number
	local sub1,sub2 = strfind(rule, "#%d+")
	local orignum = getn(self.db.account[blocktype])
	if (orignum == getn(newrule) and sub1 and sub2) then
		local pos = tonumber(strsub(rule, sub1+1, sub2+1))
		if pos and pos > 0 and pos <= orignum then
			tremove(self.db.account[blocktype], pos)
			newrule = {}
			for _,v in ipairs(self.db.account[blocktype]) do
				tinsert(newrule, v)
			end
		end
	end

	local newnum = getn(newrule)
	if orignum ~= newnum then
		self.db.account[blocktype] = newrule

		local blocktype = "RuleGroup_"..blocktype
		self:Print(format(L["RuleDeletedFormat"], L[blocktype], rule))
	else
		local blocktype = "RuleGroup_"..blocktype
		self:Print(format(L["RuleDeleteNotFoundFormat"], L[blocktype], rule))
	end
end
